@@ -1,8 +1,8 @@
 # ABSTRACT: Probes network hosts for downtime, latency, etc...
 
 package Net::Prober;
-BEGIN {
-  $Net::Prober::VERSION = '0.05';
+{
+  $Net::Prober::VERSION = '0.06';
 }
 
 
@@ -29,188 +29,77 @@ sub port_name_to_num {
     return $port;
 }
 
-sub probe_icmp {
-    my ($probe) = @_;
+sub probe_any {
+    my ($class, $args) = @_;
+    my $full_pkg = $class;
 
-    my ($host, $port, $timeout, $proto, $size) =
-        @{$probe}{qw(host port timeout proto size)};
+    # Open up for != Net::Prober::* classes
+    if ($full_pkg !~ m{::}) {
+        $full_pkg = "Net::Prober::${full_pkg}";
+    }
 
-    # icmp requires root privileges
-    my $ping_proto = ($< | $>) ? "tcp" : "icmp";
-
-    my $pinger = Net::Ping->new($ping_proto, $timeout);
-    $pinger->hires();
-    $pinger->port_number($port) if defined $port;
-
-    my ($ok, $elapsed, $ip) = $pinger->ping($host);
-    $pinger->close();
-
-    my $result = {
-        ok => $ok ? 1 : 0,
-        time => $elapsed,
-        ip => $ip,
+    eval "require $full_pkg; 1" or do {
+        Carp::croak("Couldn't load $full_pkg class: $@");
     };
 
-    return $result;
+    my $p = $full_pkg->new();
+    return $p->probe($args);
+}
+
+sub probe_icmp {
+    return probe_any('ping', @_);
+}
+
+sub probe_ping {
+    return probe_any('ping', @_);
+}
+
+sub probe_imap {
+    return probe_any('imap', @_);
+}
+
+sub probe_ssh {
+    return probe_any('ssh', @_);
+}
+
+sub probe_smtp {
+    return probe_any('smtp', @_);
 }
 
 sub probe_http {
-    my ($probe) = @_;
-
-    my ($host, $port, $timeout, $proto, $url, $expected_md5, $content_match) =
-        @{$probe}{qw(host port timeout proto url md5 match)};
-
-    my $ua = LWPx::ParanoidAgent->new();
-    $ua->agent("Net::Prober/$Net::Prober::VERSION");
-    $ua->max_redirect(0);
-    $ua->timeout($timeout);
-
-    $proto = 'http' if not defined $proto;
-
-    if ($proto eq 'http') {
-        $port ||= 80;
-    }
-    elsif ($proto eq 'https') {
-        $port ||= 443;
-    }
-
-    $url = '/' unless defined $url;
-    $url =~ s{^/+}{};
-
-    my $scheme = $port == 443 ? "https" : "http";
-    my $probe_url = "$scheme://$host:$port/$url";
-
-    my $t0 = [ Time::HiRes::gettimeofday() ];
-    my $resp = $ua->get($probe_url);
-    my $elapsed = Time::HiRes::tv_interval($t0);
-    my $content = $resp->content();
-
-    my $good = $resp->is_redirect() || $resp->is_success();
-
-    if ($good and defined $expected_md5) {
-        my $md5 = Digest::MD5::md5_hex($content);
-        if ($md5 ne $expected_md5) {
-            $good = 0;
-        }
-    }
-
-    if ($good and defined $content_match) {
-        my $match_re;
-        eval {
-            $match_re = qr{$content_match}ms;
-        } or do {
-            Carp::croak("Invalid regex for http content match '$content_match'\n");
-        };
-        if ($content !~ $match_re) {
-            $good = 0;
-        }
-    }
-
-    return {
-        ok      => $good ? 1 : 0,
-        status  => $resp->status_line,
-        time    => $elapsed,
-        content => $content,
-        md5     => $content ? Digest::MD5::md5_hex($content) : undef,
-    };
-
+    return probe_any('http', @_);
 }
 
 sub probe_tcp {
-    my ($probe) = @_;
-
-    my ($host, $port, $timeout, $proto) =
-        @{$probe}{qw(host port timeout proto)};
-
-    $port = port_name_to_num($port);
-    if (! defined $port or $port == 0) {
-        Carp::croak("Can't probe: undefined port");
-    }
-    $timeout ||= 1.0;
-
-    my $t0 = [ Time::HiRes::gettimeofday() ];
-
-    my $sock = IO::Socket::INET->new(
-        PeerAddr => $host,
-        PeerPort => $port,
-        Proto    => $proto,
-        Timeout  => $timeout,
-    );
-
-    my $good = 0;
-    my $reason;
-
-    if (! $sock) {
-        $reason = "Socket open failed";
-    }
-    else {
-        $good = $sock->connected() && $sock->close();
-        if (! $good) {
-            $reason = "Socket connect or close failed";
-        }
-    }
-
-    my $elapsed = Time::HiRes::tv_interval($t0);
-
-    my $result = {
-        ok   => $good ? 1 : 0,
-        time => $elapsed,
-        host => $host,
-        port => $port,
-    };
-
-    if (! $good) {
-        $result->{reason} = $reason;
-    }
-
-    return $result;
+    return probe_any('tcp', @_);
 }
 
 
 sub probe {
     my ($probe_type) = @_;
 
+    if (! $probe_type || ref $probe_type ne 'HASH') {
+        Carp::croak("Invalid probe data");
+    }
+
     my $host = $probe_type->{host};
     if (! defined $host or $host eq "") {
-        Carp::croak("Can't probe undefined host\n");
+        Carp::croak("Can't probe undefined host");
     }
 
-    my $proto = lc($probe_type->{proto} || 'tcp');
-    my $port  = $probe_type->{port};
+    my %probe_args = %{ $probe_type };
+    my $class = lc ($probe_args{class} || 'tcp');
 
     # Resolve port names (http => 80)
-    $port = port_name_to_num($port);
+    $probe_args{port} = port_name_to_num($probe_args{port});
 
-    my $probe = {
-        host     => $host,
-        port     => $port,
-        proto    => $proto,
-        url      => $probe_type->{url} || '/',
-        timeout  => $probe_type->{timeout} || 1.0,
-        md5      => $probe_type->{md5},
-        match    => $probe_type->{match},
-    };
-
-    my $result;
-    if ($proto eq 'http' || $proto eq 'https') {
-        $result = probe_http($probe);
-    }
-    elsif ($proto eq 'tcp') {
-        $result = probe_tcp($probe);
-    }
-    elsif ($proto eq 'icmp') {
-        $result = probe_icmp($probe);
-    }
-    else {
-        Carp::croak("Not implemented $proto probe yet");
-    }
+    my $result = probe_any($class, \%probe_args);
 
     return $result;
 }
 
 1;
 
-
 __END__
 =pod
 
@@ -220,14 +109,14 @@ Net::Prober - Probes network hosts for downtime, latency, etc...
 
 =head1 VERSION
 
-version 0.05
+version 0.06
 
 =head1 SYNOPSIS
 
     use Net::Prober;
 
     my $result = Net::Prober::probe({
-        proto => 'tcp',
+        class => 'tcp',
         port => 'ssh',
         host => 'localhost',
         timeout => 0.5,
@@ -243,7 +132,7 @@ version 0.05
     # or...
 
     my $result = Net::Prober::probe({
-        proto => 'http',
+        protocol => 'http',
         host => 'www.opera.com',
         url => '/browser',
         match => 'Faster',
@@ -316,7 +205,7 @@ the Varnish probe config block:
         "1.2.3.4" : {
             "datacenter" : "norway1",
             "probe" : {
-                "proto": "tcp",
+                "protocol": "tcp",
                 "port" : "8432",
                 "timeout" : 1.0,
             },
@@ -347,7 +236,7 @@ Allowed hash keys are:
 
 =over 4
 
-=item C<proto>
+=item C<protocol>
 
 What type of probe you want to run.
 Can be any of C<tcp>, C<http>, C<icmp>.
@@ -376,7 +265,7 @@ Returns the results as hashref. Example:
     my $result = Net::Prober::probe({
         host => 'localhost',
         port => 'ssh',
-        proto => 'tcp',
+        protocol => 'tcp',
         timeout => 0.5,
     });
 
@@ -429,7 +318,7 @@ to send a ping packet to the given host and port.
 
 C<size> of ping packets is not currently supported.
 
-C<proto> is an additional key that allows to specify
+C<protocol> is an additional key that allows to specify
 whether the pings should be sent via UDP or TCP. Remember
 that to send ICMP UDP packets you need root privileges.
 
@@ -442,7 +331,7 @@ Cosimo Streppone <cosimo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2011 by Cosimo Streppone.
+This software is copyright (c) 2012 by Cosimo Streppone.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
